#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define RQ_SET_LED    0
#define RQ_GET_SWITCH 1

/* -------------------------- TWI - Two-wire Interface --------------------- */
void TWIInit(uint8_t add,uint8_t mul){
    TWSR = mul;
    TWBR = add;
    TWCR = 1<<TWEN;
}

void TWIStart(){
    TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);
    //while ((TWCR & (1 << TWINT)) == 0);
    _delay_ms(50);
}

void TWIStop(){
    TWCR = (1<<TWINT)|(1<<TWSTO)|(1<<TWEN);
    _delay_ms(50);
}

void TWIWrite(uint8_t data){
    TWDR = data;
    TWCR = (1<<TWINT)|(1<<TWEN);
    //while ((TWCR & (1 << TWINT)) == 0);
    _delay_ms(50);
}

uint8_t TWIReadACK()
{
    TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWEA);
    //while ((TWCR & (1<<TWINT)) == 0);
    _delay_ms(50);
    return TWDR;
}

uint8_t TWIRead()
{
    TWCR = (1<<TWINT)|(1<<TWEN);
    //while ((TWCR & (1<<TWINT)) == 0);
    _delay_ms(50);
    return TWDR;
}

uint8_t TWIGetStatus()
{
    uint8_t status;
    status = TWSR & 0xF8;
    _delay_ms(50);
    return status;
}

void ADCInit(){
    return 0;
    ADMUX |= (1<<REFS0);
    ADCSRA |= (1<<ADPS2)|(1<<ADPS1);
    ADCSRA |= (ADEN);
}
void ADCChan(uint8_t chan){
    ADMUX &= ~(0b11111);
    ADMUX |= (0b11111) & chan;
}
uint16_t ADCRead(){
    ADCSRA |= (1<<ADSC);
    while(!(ADCSRA & (1<<ADIF)));
    ADCSRA |= (1<<ADIF);
    return ADC;
}
uint16_t ADCRead2(){
    ADCSRA = 0b11000110;
    while(ADCSRA & (1<<ADSC));
    return ADC;
}

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */
int sz,cpi;
uint8_t cp[50];
int send(uint32_t data,int size){
    int i=0;
    cp[cpi++]=size;
    for(;i<size;i++){
        cp[cpi+i] = ((uint8_t)data);
        data >>= 8;
    }
    cpi+=size;
    return 0;
}

int delayget=1;
int delayclr=1;
usbMsgLen_t usbResponse(uint8_t data[8])
{
    int i,j;uint8_t t1,t2;
    usbRequest_t *rq = (void *)data;
    uint8_t led_val = rq->wValue.bytes[0];
    uint8_t led_no  = rq->wIndex.bytes[0];

    switch (rq->bRequest)
    {
        case 0:
            if (led_val)
                PORTC |= (1<<led_no);
            else
                PORTC &= ~(1<<led_no);
            return 0;
        case 1:
                return send((PINC & (1<<led_no)) == 0 ?1:0, 1);
        case 2:
            ADMUX = 0b01000000; 
            ADMUX |= led_no;
            ADCSRA = 0b11000110;
            while ((ADCSRA & (1<<ADSC)));
            return send(ADC,2);
        case 3:
            return send(DDRC,1);
        case 4:
            DDRC = led_no;
            return 0;
        
#define setport(PORT) \
            if (led_val==0) PORT = led_no;\
            else if (led_val==1) PORT |= led_no;\
            else if (led_val==2) PORT &= led_no;\
            else if (led_val==3) send(PORT,1);\
            return 0;
            
        //case 11:
        //    setport(PORTA);
        case 12:
            setport(PORTB);
        case 13:
            setport(PORTC);
        case 14:
            setport(PORTD);
            
        //case 21:
        //    setport(DDRA);
        case 22:
            setport(DDRB);
        case 23:
            setport(DDRC);
        case 24:
            setport(DDRD);
            
        //case 41:
        //    setport(PINA);
        case 42:
            setport(PINB);
        case 43:
            setport(PINC);
        case 44:
            setport(PIND);
            
            
        case 30:
            /// I/O Extender
            
            if(led_no&0x80){
                TWIStart();
                    send(TWIGetStatus(),1);
                TWIWrite((led_no<<1)&0xFF);
                    send(TWIGetStatus(),1);
                TWIWrite(led_val);
                    send(TWIGetStatus(),1);
                TWIStop();
                    return send(0,1);
            }
            else{
            TWIStart();
            if(TWIGetStatus()!=0x8)
                return send(-1,1)+send(TWIGetStatus(),1);
            TWIWrite((led_no<<1)&0xFF);
            if(TWIGetStatus()!=0x18)
                return send(-2,1)+send(TWIGetStatus(),1);
            TWIWrite(led_val);
            if(TWIGetStatus()!=0x28)
                return send(-3,1)+send(TWIGetStatus(),1);
            TWIStop();
            }
            return send(0,1);
        case 31:
            
            if(led_no&0x80){
                TWIStart();
                send(TWIGetStatus(),1);
                TWIWrite(((led_no&0x7F)<<1)|1);
                send(TWIGetStatus(),1);
                led_val=TWIRead();
                send(TWIGetStatus(),1);
                TWIStop();
                return send(led_val,1);
            }
            else{
            TWIStart();
            if(TWIGetStatus()!=0x8)
                return send(-1,2)+send(TWIGetStatus(),1);
            TWIWrite(((led_no&0x7F)<<1)|1);
            if(TWIGetStatus()!=0x18)
                return send(-2,2)+send(TWIGetStatus(),1);
            led_val = TWIRead();
            if(TWIGetStatus()!=0x58)
                return send(-3,2)+send(TWIGetStatus(),1);
            TWIStop();
            }
            return send(led_val,1);
        case 32:
            TWIInit(led_no,led_val);
            return 0;
        case 33:
            TWIStart();
            return send(TWIGetStatus(),1);
        case 34:
            TWIWrite(led_no);
            return send(TWIGetStatus(),1);
        case 35:
            led_val = TWIRead();
            return send(TWIGetStatus(),1)+send(led_val,1);
        case 36:
            TWIStop();
            return send(TWIGetStatus(),1);
        case 37:
            return send(TWIGetStatus(),1);
        case 38:
            led_val = TWIReadACK();
            return send(TWIGetStatus(),1)+send(led_val,1);
            
        case 60: //Init
            ADCInit();
            return 0;
        case 61: // Read capacitive
            t1 = (1<<led_no);
            t2 = ~t1;
            
            for(i=0;i<led_val;i++){//Collect multiple sample
                
                //Discharge
                PORTC &= t2;
                DDRC |= t1; // Connect to ground
                for(j=delayclr;j>0;j--)_delay_us(10);
                ADCChan(led_no);
                ADCRead2(); // Clear
                DDRC &= t2; // Set to input
                
                //Discharge2
                ADCChan(0b11111);
                ADCRead2(); // Clear
                
                //Pull up
                PORTC |= t1;
                for(j=delayget;j>0;j--)_delay_us(10);
                PORTC &= t2;
                
                //Read
                ADCChan(led_no);
                send(ADCRead2(),2);
            }
            return 0;
        case 62:
            ADCChan(led_no);
            return send(ADCRead2(),2);
        case 63:
            delayget = (((int)led_no)<<8) + led_val;
            return 0;
        case 64:
            delayclr = (((int)led_no)<<8) + led_val;
            return 0;
    }

    return 0;   /* default for not implemented requests: return no data back to host */
}
usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    sz = 0; cpi = 0;
    usbResponse(data);
    //int i=0;
    //for(;i<sz&&i<29;i++)
    //    cp[i+1] = ((uint8_t*)usbMsgPtr)[i];
    cp[cpi++]=255;
    usbMsgPtr = (void*) cp;
    return cpi;
}

/* ------------------------------------------------------------------------- */

int main(void)
{
    //DDRC  = (1<<PC0)|(1<<PC1)|(1<<PC2)|(1<<PC5); // PC0..PC2 = Output, PC3..PC5 = Input
    //PORTC = (1 << PC3);                 // Enable pull-up at PC3

    usbInit();

    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();

    /* main event loop */
    for(;;)
    {
        usbPoll();
    }

    return 0;
}

/* ------------------------------------------------------------------------- */
