import usb
import struct

####################################
class McuBoard:
    '''
    Generic class for accessing Practicum MCU board via USB connection.
    '''

    ################################
    def __init__(self):
        for bus in usb.busses():
            for dev in bus.devices:
                if (dev.idVendor,dev.idProduct) == (0x16c0,0x05dc):
                    self.handle = dev.open()
                    return
        raise Exception("MCU board not found!")

    ################################
    def usb_write(self, request, data=[], index=0, value=0):
        '''
        Send data output to the USB device (i.e., MCU board)
           request: request number to appear as bRequest field on the USB device
           index: 16-bit value to appear as wIndex field on the USB device
           value: 16-bit value to appear as wValue field on the USB device
        '''
        reqType = usb.TYPE_VENDOR | usb.RECIP_DEVICE | usb.ENDPOINT_OUT
        self.handle.controlMsg(
                reqType, request, data, value=value, index=index)

    ################################
    def usb_read(self, request, length=1, index=0, value=0):
        '''
        Request data input from the USB device (i.e., MCU board)
           request: request number to appear as bRequest field on the USB device
           length: number of bytes to read from the USB device
           index: 16-bit value to appear as wIndex field on the USB device
           value: 16-bit value to appear as wValue field on the USB device

        If successful, the method returns a tuple of length specified
        containing data returned from the MCU board.
        '''
        reqType = usb.TYPE_VENDOR | usb.RECIP_DEVICE | usb.ENDPOINT_IN
        buf = self.handle.controlMsg(
                reqType, request, length, value=value, index=index)
        return buf

    def usb(self, request, index=0, value=0):
        reqType = usb.TYPE_VENDOR | usb.RECIP_DEVICE | usb.ENDPOINT_IN
        resp = self.handle.controlMsg(
                                     reqType, request, 256, value=value, index=index)
        i = 0
        #return resp
        fmt = {0:'', 1:'B', 2:'H', 4:'L', 8:'Q'}
        result = []
        while resp[i] != 255 and i < 256:
            sz = resp[i]
            #print(str(sz)+'='+repr(bytes(resp[i+1:i+sz+1])))
            k = struct.unpack(('<'+fmt[sz]),bytes(resp[i+1:i+sz+1]))
            if len(fmt) > 0:
                result.append(k[0])
            i += sz + 1
        return result

    port = {
        'PORTB':12,
        'PORTC':13,
        'PORTD':14,
        'DDRB':22,
        'DDRC':23,
        'DDRD':24,
        'PINB':42,
        'PINC':43,
        'PIND':44,
    };
    command = {
        'write':0,
        'assign':0,
        'set':0,
        'add':1,
        'or':1,
        'mask':2,
        'sub':2,
        'and':2,
        'get':3,
        'read':3,
        0:0,
        1:1,
        2:2,
        3:3,
    };

    def io(self,cmd,port,data=0):
        return self.usb(self.port[port],data,self.command[cmd])

    def setBit(self,port,position,val):
        if val:
            self.io('add',port,1<<position)
        elif val:
            self.io('sub',port,0xFF^(1<<position))

    def ioCon(self):
        i=input()
        while i!='exit':
            i=i.split(' ')
            if(len(i)>2): i[2]=eval(i[2])
            else: i[2:]=[0]
            if(len(i)>3): i[3]=eval(i[3])
            if(i[0].endswith('bit')):
                self.setBit(i[1],i[2],(i[3]))
            else:
                print(self.io(i[0],i[1],i[2]))
            i=input()
                
